@@ -1,9 +1,10 @@
 /*
-**  Str - String Library
-**  Copyright (c) 1999-2001 Ralf S. Engelschall <rse@engelschall.com>
+**  OSSP str - String Handling
+**  Copyright (c) 1999-2005 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1999-2005 The OSSP Project <http://www.ossp.org/>
 **
-**  This file is part of Str, a string handling and manipulation 
-**  library which can be found at http://www.engelschall.com/sw/str/.
+**  This file is part of OSSP str, a string handling and manipulation
+**  library which can be found at http://www.ossp.org/pkg/lib/str/.
 **
 **  Permission to use, copy, modify, and distribute this software for
 **  any purpose with or without fee is hereby granted, provided that
@@ -34,7 +35,7 @@
  *  are met:
  *
  *  1. Redistributions of source code must retain the above copyright
- *     notice, this list of conditions and the following disclaimer. 
+ *     notice, this list of conditions and the following disclaimer.
  *
  *  2. Redistributions in binary form must reproduce the above copyright
  *     notice, this list of conditions and the following disclaimer in
@@ -107,7 +108,7 @@ typedef unsigned long      u_quad_int;
 #define FLOAT_DIGITS    6
 #define EXPONENT_LENGTH 10
 
-/* NUM_BUF_SIZE is the size of the buffer used for arithmetic 
+/* NUM_BUF_SIZE is the size of the buffer used for arithmetic
    conversions. This is a magic number; do NOT decrease it! */
 #define NUM_BUF_SIZE    512
 #define NDIG            80
@@ -120,7 +121,7 @@ typedef unsigned long      u_quad_int;
 #define isinf(d) (0)
 #endif
 
-/* 
+/*
  * Convert decimal number to its string representation. The number of
  * digits is specified by ndigit decpt is set to the position of the
  * decimal point sign is set to 0 for positive, 1 for negative. buf must
@@ -133,12 +134,31 @@ typedef unsigned long      u_quad_int;
 #define str_fcvt(arg,ndigits,decpt,sign,buf) \
         str_cvt((arg), (ndigits), (decpt), (sign), 0, (buf))
 
+/* inlined modf(3) to avoid dependency to external libm on systems
+   (like Tru64, QNX, etc) where modf(3) is not part of libc. */
+static double
+str_modf(
+    double arg,
+    double *iptr)
+{
+    double fraction;
+    double integral;
+    long trunc;
+
+    trunc = (long)arg;
+    integral = (double)trunc;
+    fraction = arg - integral;
+    if (iptr != NULL)
+        *iptr = integral;
+    return fraction;
+}
+
 static char *
 str_cvt(
-    double arg, 
-    int ndigits, 
-    int *decpt, 
-    BOOL *sign, 
+    double arg,
+    int ndigits,
+    int *decpt,
+    BOOL *sign,
     int eflag,
     char *buf)
 {
@@ -155,14 +175,14 @@ str_cvt(
         *sign = TRUE;
         arg = -arg;
     }
-    arg = modf(arg, &fi);
+    arg = str_modf(arg, &fi);
     p1 = &buf[NDIG];
 
     /* Do integer part */
     if (fi != 0) {
         p1 = &buf[NDIG];
         while (fi != 0 && p1 > &buf[0]) {
-            fj = modf(fi / 10, &fi);
+            fj = str_modf(fi / 10, &fi);
             *--p1 = (int)((fj + .03) * 10) + '0';
             r2++;
         }
@@ -185,7 +205,7 @@ str_cvt(
     }
     while (p <= p1 && p < &buf[NDIG]) {
         arg *= 10;
-        arg = modf(arg, &fj);
+        arg = str_modf(arg, &fj);
         *p++ = (int) fj + '0';
     }
     if (p1 >= &buf[NDIG]) {
@@ -214,9 +234,9 @@ str_cvt(
 
 static char *
 str_gcvt(
-    double number, 
-    int ndigit, 
-    char *buf, 
+    double number,
+    int ndigit,
+    char *buf,
     BOOL altform)
 {
     BOOL sign;
@@ -298,7 +318,7 @@ str_gcvt(
 }
 
 /*
- * Convert a string to decimal value 
+ * Convert a string to decimal value
  */
 #define NUM(c) ((c) - '0')
 #define STR_TO_DEC(str, num) {    \
@@ -308,7 +328,7 @@ str_gcvt(
         num += NUM(*str++) ;      \
     }                             \
 }
-     
+
 /*
  * This macro does zero padding so that the precision requirement is
  * satisfied. The padding is done by adding '0's to the left of the
@@ -323,7 +343,7 @@ str_gcvt(
     }
 
 /*
- * This macro does padding. 
+ * This macro does padding.
  * The padding is done by printing the character ch.
  */
 #define PAD(width, len, ch) \
@@ -357,9 +377,9 @@ str_gcvt(
  */
 static char *
 conv_10(
-    register long_int num, 
+    register long_int num,
     register BOOL is_unsigned,
-    register BOOL *is_negative, 
+    register BOOL *is_negative,
     char *buf_end,
     register int *len)
 {
@@ -372,7 +392,7 @@ conv_10(
     }
     else {
         *is_negative = (num < 0);
-        /* On a 2's complement machine, negating the most negative integer 
+        /* On a 2's complement machine, negating the most negative integer
            results in a number that cannot be represented as a signed integer.
            Here is what we do to obtain the number's magnitude:
                 a. add 1 to the number
@@ -399,9 +419,9 @@ conv_10(
 
 static char *
 conv_10_quad(
-    quad_int num, 
+    quad_int num,
     register BOOL is_unsigned,
-    register BOOL *is_negative, 
+    register BOOL *is_negative,
     char *buf_end,
     register int *len)
 {
@@ -414,7 +434,7 @@ conv_10_quad(
     }
     else {
         *is_negative = (num < 0);
-        /* On a 2's complement machine, negating the most negative integer 
+        /* On a 2's complement machine, negating the most negative integer
            result in a number that cannot be represented as a signed integer.
            Here is what we do to obtain the number's magnitude:
                 a. add 1 to the number
@@ -446,12 +466,12 @@ conv_10_quad(
  */
 static char *
 conv_fp(
-    register char format, 
+    register char format,
     register double num,
-    BOOL add_dp, 
-    int precision, 
+    BOOL add_dp,
+    int precision,
     BOOL *is_negative,
-    char *buf, 
+    char *buf,
     int *len)
 {
     register char *s = buf;
@@ -507,7 +527,7 @@ conv_fp(
         *s++ = format;          /* either e or E */
         decimal_point--;
         if (decimal_point != 0) {
-            p = conv_10((long_int) decimal_point, FALSE, &exponent_is_negative, 
+            p = conv_10((long_int) decimal_point, FALSE, &exponent_is_negative,
                         &temp[EXPONENT_LENGTH], &t_len);
             *s++ = exponent_is_negative ? '-' : '+';
             /* Make sure the exponent has at least 2 digits */
@@ -544,10 +564,10 @@ static const char upper_digits[] = "0123456789ABCDEF";
 
 static char *
 conv_p2(
-    register u_long_int num, 
+    register u_long_int num,
     register int nbits,
-    char format, 
-    char *buf_end, 
+    char format,
+    char *buf_end,
     register int *len)
 {
     register int mask = (1 << nbits) - 1;
@@ -564,10 +584,10 @@ conv_p2(
 
 static char *
 conv_p2_quad(
-    u_quad_int num, 
+    u_quad_int num,
     register int nbits,
-    char format, 
-    char *buf_end, 
+    char format,
+    char *buf_end,
     register int *len)
 {
     register int mask = (1 << nbits) - 1;
@@ -582,14 +602,14 @@ conv_p2_quad(
     return p;
 }
 
-/* 
+/*
  * str_vformat(), the generic printf-style formatting routine
  * and heart of this piece of source.
  */
-int 
+int
 str_vformat(
-    str_vformat_t *vbuff, 
-    const char *fmt, 
+    str_vformat_t *vbuff,
+    const char *fmt,
     va_list ap)
 {
     register char *sp;
@@ -883,7 +903,7 @@ str_vformat(
                     else {
                         /* use &num_buf[1], so that we have room for the sign */
                         s = conv_fp(*fmt, fp_num, alternate_form,
-                                    (adjust_precision == FALSE) ? FLOAT_DIGITS : precision, 
+                                    (adjust_precision == FALSE) ? FLOAT_DIGITS : precision,
                                     &is_negative, &num_buf[1], &s_len);
                         if (is_negative)
                             prefix_char = '-';
@@ -958,7 +978,7 @@ str_vformat(
                     break;
 
                 /*
-                 * Pointer argument type. 
+                 * Pointer argument type.
                  */
                 case 'p':
 #if defined(SIZEOF_LONG_LONG) && (SIZEOF_LONG_LONG == SIZEOF_VOID_P)
@@ -991,8 +1011,8 @@ str_vformat(
                 default:
                     s = NULL;
                     if (vbuff->format != NULL) {
-                        s = vbuff->format(vbuff, 
-                            &prefix_char, &pad_char, &s_len, 
+                        s = vbuff->format(vbuff,
+                            &prefix_char, &pad_char, &s_len,
                             num_buf, NUM_BUF_SIZE, extinfo, *fmt, ap);
                     }
                     if (s == NULL) {
@@ -1021,7 +1041,7 @@ str_vformat(
             }
 
             /*
-             * Print the string s. 
+             * Print the string s.
              */
             for (i = s_len; i != 0; i--) {
                 INS_CHAR(*s, sp, bep, cc);
@@ -1127,5 +1147,5 @@ char *str_aprintf(const char *fmt, ...)
         return NULL;
     return s;
 }
-#endif 
+#endif
 
